"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var babel_plugin_macros_1 = require("babel-plugin-macros");
var path = require("path");
var fs = require("fs");
var countLines = function (text) { return text.split("\n").length; };
var countWords = function (text) {
    return text.length > 0 ? text.trim().split(/\s+/).length : 0;
};
var macro = function (_a) {
    var references = _a.references, state = _a.state, babel = _a.babel;
    var t = babel.types;
    var applyCurrentFile = function (fn) { return function (nodePath) {
        if (t.isIdentifier(nodePath.node)) {
            nodePath.replaceWith(t.numericLiteral(fn("/")));
        }
    }; };
    var applyInAnotherFile = function (currentFilename) { return function (fn) { return function (nodePath) {
        var parent = nodePath.parent;
        if (t.isCallExpression(parent)) {
            if (parent.arguments.length !== 1) {
                throw new Error("Can only call linesIn with a single string argument");
            }
            var filenameNode = parent.arguments[0];
            if (!t.isStringLiteral(filenameNode)) {
                throw new Error("Argument to linesIn must be a string literal");
            }
            var absolutePath = path.resolve(path.dirname(currentFilename), filenameNode.value);
            nodePath.parentPath.replaceWith(t.numericLiteral(fn(absolutePath)));
        }
    }; }; };
    var contentsCache = {
        "/": state.file.code,
    };
    var readFile = function (filename) {
        var contents = contentsCache[filename];
        if (contents == null) {
            contents = fs.readFileSync(filename, "utf-8");
            contentsCache[filename] = contents;
        }
        return contents;
    };
    var getLinesInFile = function (filename) {
        return countLines(readFile(filename));
    };
    var getWordsInFile = function (filename) {
        return countWords(readFile(filename));
    };
    var currentFilename = state.file.opts.filename;
    // lines in current file
    (references.lines || []).map(applyCurrentFile(getLinesInFile));
    // lines in another file
    (references.linesIn || []).map(applyInAnotherFile(currentFilename)(getLinesInFile));
    // words in current file
    (references.words || []).map(applyCurrentFile(getWordsInFile));
    // words in another file
    (references.wordsIn || []).map(applyInAnotherFile(currentFilename)(getWordsInFile));
};
exports.lines = null;
exports.linesIn = null;
exports.words = null;
exports.wordsIn = null;
exports.default = babel_plugin_macros_1.createMacro(macro);
